# unmachined_areas.py
# billtrondsen@yahoo.com
# rev 21-Mar-2015 - release

ToolList = (0.125, 0.25, 0.5, 0.75, 1)

########  IMPORTS
import sys
import clr
clr.AddReference("System.Drawing")
clr.AddReference("System.Windows.Forms")
from System.Drawing import Point
from System.Windows.Forms import (Application, BorderStyle, Button,
	CheckBox, Form, Label, Panel, RadioButton, TextBox, ListBox)

#############################################################
######  WINDOWS FORM DEFINITION
#############################################################

class CBWindowsFormDemo(Form):

    def __init__(self):
        self.Text = 'UnmachinedAreas.py'
		self.Height = 200
		self.Width = 300
		
######  DEFINE CONTROLS
		
        self.label1 = Label()
        self.label1.Text = "Tool Diameter"
        self.label1.Location = Point(40, 20)
        self.label1.AutoSize = True

        self.listBox1 = ListBox()
		self.listBox1.Text = "My List Box"
		self.listBox1.Location = Point(50,40)
		self.listBox1.Height = 100
		self.listBox1.Width = 50
        self.listBox1.Items.AddRange(ToolList)
		
        self.radio1 = RadioButton()
		self.radio1.Name = "radio1"
        self.radio1.Text = "Inside Cut"
        self.radio1.Location = Point(150, 40)
        self.radio1.Checked = True

        self.radio2 = RadioButton()
		self.radio2.Name = "radio2"
        self.radio2.Text = "Outside Cut"
        self.radio2.Location = Point(150, 60)

        self.button1 = Button()
		self.button1.Name = "button1"
        self.button1.Text = "SUBMIT"
        self.button1.Location = Point(150, 100)
#        self.button1.Click += self.buttonPressed

######  ADD CONTROLS

        self.Controls.Add(self.label1)
        self.Controls.Add(self.listBox1)	
        self.Controls.Add(self.radio1)
		self.Controls.Add(self.radio2)
        self.Controls.Add(self.button1)

        self.AcceptButton = self.button1  #### Set AcceptButton for the form
        self.CancelButton = self.button1  #### Set the CancelButton for the form

######  FUNCTIONS
		
#    def update(self, sender, event):  #Called when the AcceptButton is pressed
#        print 'The AcceptButton %s was pressed.'% sender.Name

#    def reset(self, sender, event):   #Called when the CancelButton is pressed
#        print 'The CancelButton %s was pressed.'% sender.Name

#    def buttonPressed(self, sender, args):
#        print 'The button %s has been pressed.'% sender.Name

#	def checkedChanged(self, sender, args):
#		print 'The value of ',sender.Name, ' changed to ',sender.Checked

Application.EnableVisualStyles()
form = CBWindowsFormDemo()
form.listBox1.SelectedIndex = 0  #Selects the first item in the list as the default
Form.ShowDialog(form)

#print form.listBox1.SelectedItem.ToString()
#print form.radio1.Checked.ToString()
#print form.radio2.Checked.ToString()

ToolDiameter = form.listBox1.SelectedItem
print ("ToolDiameter = ")+ToolDiameter.ToString()

if form.radio1.Checked == True:
  InOrOut = "In"
elif form.radio2.Checked == True:
  InOrOut = "Out"

print ("InOrOut = ")+InOrOut.ToString()

####################################################
#### MAIN CODE
####################################################

CamBamUI.MainUI.UndoBuffer.AddUndoPoint("Undo Here")
CamBamUI.MainUI.UndoBuffer.Add(doc.ActiveLayer.Entities)

###Store selected entities in a ShapeList
ScreenSelections = ShapeList()
ScreenSelections.AddSelectedEntities(view)
#print ScreenSelections
#print ScreenSelections.Count

###Make a copy and convert to Regions
SelectionClone = ScreenSelections.Clone()
print ("SelectionClone.Count = ")+SelectionClone.Count.ToString()

####################################################
#### INSIDE PATH
####################################################

if InOrOut == "In":
  SelectionClone = SelectionClone.DetectRegions(True)
  print ("SelectionClone.Count = ")+SelectionClone.Count.ToString()+(" After Regions") 
  #doc.Add(SelectionClone.ToEntities())

  ###Create Region Fill using Inside/Outside Offsets
  ###Margin = Tool Diameter and Stepover = Large Value to create only one offset
  ToolOffset = RegionFiller_InsideOutsideOffsets(SelectionClone, 100000, ToolDiameter)
  ToolOffset.GenerateFill()
  
  ToolOffsetPolys = ToolOffset.FillShapes.ToPolylines()
  print ("ToolOffsetPolys.Count = ")+ToolOffsetPolys.Count.ToString()
  ToolOffsetPolys = ShapeList(ToolOffsetPolys,False).DetectRegions(True)
  #doc.Add(ToolOffsetPolys.ToEntities())

  ###ToolOffsetPolys toward the Original Path - this is the Cut Edges
  CutEdges = ToolOffsetPolys.Offset(ToolDiameter)
  CutEdges = CutEdges.DetectRegions(True)
  CutEdges = CutEdges.Union()
  #doc.Add(CutEdges.ToEntities())

  ###Subtract the Cut Path from the Original Path to leave the Unmachined Areas
  SC2 = SelectionClone.Clone().DetectRegions(True)
  CE2 = CutEdges.Clone().DetectRegions(True)
  UnmachinedArea = SC2.Subtract(CE2)
  if doc.HasLayer("Unmachined Areas") == False:
    doc.CreateLayer("Unmachined Areas")
  doc.SetActiveLayer("Unmachined Areas")
  doc.Add(UnmachinedArea.ToEntities())

####################################################
#### OUTSIDE PATH
####################################################

if InOrOut == "Out":
  Border = SelectionClone.Offset(ToolDiameter)
  Border = Border.ToPolylines()
  Border = ShapeList(Border,False)
  print ("Border.Count = ")+Border.Count.ToString()+(" Polylines")
  Border = Border.Union()
  print ("Border.Count = ")+Border.Count.ToString()+(" Union")
  #doc.Add(Border.ToEntities())
  SelectionClone = SelectionClone.ToPolylines()
  SelectionClone = ShapeList(SelectionClone,False)
  SelectionClone.AddEntities(Border.ToEntities())
  print ("SelectionClone.Count = ")+SelectionClone.Count.ToString()+(" Plus Border")
  SelectionClone = SelectionClone.DetectRegions(True)
  print ("SelectionClone.Count = ")+SelectionClone.Count.ToString()+(" After Regions") 
  #doc.Add(SelectionClone.ToEntities())

  ###Create Region Fill using Inside/Outside Offsets
  ###Margin = Tool Diameter and Stepover = Large Value to create only one offset
  ToolOffset = RegionFiller_InsideOutsideOffsets(SelectionClone, 100000, ToolDiameter)
  ToolOffset.GenerateFill()
  print ("Generated ToolOffset")
  ToolOffsetPolys = ToolOffset.FillShapes.ToPolylines()
  print ("ToolOffsetPolys.Count = ")+ToolOffsetPolys.Count.ToString()
  ToolOffsetPolys = ShapeList(ToolOffsetPolys,False)
  ToolOffsetPolys.AddEntities(Border.ToEntities())
  ToolOffsetPolys = ToolOffsetPolys.DetectRegions(True)
  #doc.Add(ToolOffsetPolys.ToEntities())

  ###ToolOffsetPolys toward the Original Path - this is the Cut Edges
  CutEdges = ToolOffsetPolys.Offset(-ToolDiameter)
  CutEdges = CutEdges.DetectRegions(True)
  CutEdges = CutEdges.Union()
  CutEdges = CutEdges.DetectRegions(True)
  #doc.Add(CutEdges.ToEntities())

  ###Subtract the Cut Path from the Original Path to leave the Unmachined Areas
  SC2 = ScreenSelections.Clone().DetectRegions()
  CE2 = CutEdges.Clone()
  UnmachinedArea = CE2.Subtract(SC2)
  if doc.HasLayer("Unmachined Areas") == False:
    doc.CreateLayer("Unmachined Areas")
  doc.SetActiveLayer("Unmachined Areas")
  doc.Add(UnmachinedArea.ToEntities())